function [ loads,ref ] = factorsClassify( A,B,C,varargin )
%fcactorsClassify Summary of this function goes here
%Step 2 of Parallel Factor Framework-Clustering Analysis (PFFCA):
%componments formation
% ----------------------INPUT---------------------
%
% A, B, C              Estimate of loadings from step 1 of PFFCA.  
%
% ----------------OPTIONAL INPUT---------------------
% 
% constantThreshold     constantThreshold is chosen as an indicator to 
%                       determine whether a factor's scores vary widely.
%                       When factor's average/std exceeds the constantThreshold,
%                       this factor's scores vary widely. Default value is 0.3.
% 
% pcaThreshold          Determine the number of principal components which 
%                       initFactor will be equal to minFactor. Default value is 1.
% 
% ---------------------OUTPUT---------------------
%
% loads        3D data of each components. 
%
% ref          reference imformation.
%

% Chen Qian
% University of Science and Technology of China
% E-mail qianc@mail.ustc.edu.cn

%Default value of parameters
constantThreshold=0.3;
rThreshold=1;
pcaThreshold=0.95;
loadsNum=0;

while length(varargin)>=2,
    prop =varargin{1};
    val=varargin{2};
    varargin=varargin(3:end);
    eval([prop '=val;']);     
end

% calculate wA
wA=abs(std(A,0,1)./(mean(A,1)));
I=find(wA<constantThreshold);
rA=corr(A);
constantI=[];
for i=1:length(I)
    temp=rA(I(i),:);
    I2=find(temp>=rThreshold);
    if  isempty(setdiff(I2,I))
        constantI=[constantI,I2];
    end
end
constantI=unique(constantI);
Ax=A;Bx=B;Cx=C;
if constantI
    constantLoad= combineFactors( constantI,A,B,C );
    Ax(:,constantI)=[];Bx(:,constantI)=[];Cx(:,constantI)=[];
end

if ~isempty(I) && I==constantI
    warning('ConstantThreshold might be inappropriate.');
    warning('Please try [ loads,ref ] = factorsClassify( A,B,C,''constantThreshold'',constantThreshold );');
end

% determine component number
if loadsNum==0    
    [PC,score,latent]=princomp(Ax);
    temp=0;
    sumLatent=sum(latent);
    for i=1:length(latent)
        temp=temp+latent(i)/sumLatent;
        if temp>=pcaThreshold
            clusterNum=i;
            break;
        end
    end
    ref.PCA.PC=PC;
    ref.PCA.score=score;
    ref.PCA.latent=latent;
else
    if constantI
        clusterNum=loadsNum-1;
    else
        clusterNum=loadsNum;
    end
end
    
%cluster analysis
if clusterNum==1
    r.pre_result=ones(size(Ax,2),1);
    r.result{1}=1:size(Ax,2);
else
    r=cluster_data(Ax,clusterNum);
end

%combine Factors
loads=cell(clusterNum,1);
for i=1:clusterNum
    BC= combineFactors( r.result{i},Ax,Bx,Cx );
    loads{i}=BC;
end

if constantI
    loadsNum=clusterNum+1;
    loads{loadsNum}=constantLoad;
    ref.classifyResult=cell(loadsNum,1);
    temp=r.pre_result;
    [m,n]=size(temp);
    if m>n
        temp=temp';
    end
    for i=1:length(constantI)
        if constantI(i)==1
            temp=[loadsNum,temp(1,:)];
        elseif constantI(i)==length(temp)+1
            temp=[temp(1,:),loadsNum];
        else
            temp=[temp(1,1:constantI(i)-1),loadsNum,temp(1,constantI(i):end)];
        end
    end
    for j=1:length(temp)
        ref.classifyResult{temp(j)}=[ ref.classifyResult{temp(j)} j];
    end
else
    loadsNum=clusterNum;
    ref.classifyResult=r.result;
end
ref.wA=wA;
ref.cluster=r;
ref.loadNum=loadsNum;


function [ result ] = cluster_data( A,cluster_number )
%CLUSTERDATA Summary of this function goes here
%   Detailed explanation goes here
result.number=cluster_number;
if cluster_number==size(A,2)
    result.pre_result=1:size(A,2);
else
    distence=acos(corr(A));
    distence(logical(eye(size(distence))))=0;
    distence=abs(sin(distence));
    Y=squareform(distence);
    [result.linkage, result.method, result.cophenet]=best_linkage(Y);
    result.pre_result=cluster(result.linkage,'maxclust',cluster_number);
end

result.result=cell(result.number,1);
for i=1:size(A,2)
    result.result{result.pre_result(i)}=[ result.result{result.pre_result(i)} i];
end

function [ Z,method,co ] = best_linkage( Y )
%BESTLINKAGE Summary of this function goes here
%   Detailed explanation goes here
mdata={'single' 'complete' 'average' 'weighted' 'centroid' 'ward'};
co=0;
for i=1:size(mdata,2)
    tempZ=linkage(Y,mdata{i});
    tempco=cophenet(tempZ,Y);
    if co<tempco
        co=tempco;
        method=mdata{i};
        Z=tempZ;
    end
end

function [ load ] = combineFactors( factorsClass,A,B,C )
%COMBINEFACTOR Summary of this function goes here
%   Detailed explanation goes here

load=zeros(size(A,1),size(B,1),size(C,1));
for k=1:max(size(factorsClass))
    l=factorsClass(k);
    tempBC=B(:,l)*C(:,l)';
    for j=1:size(A,1)
        temp(:,:)=load(j,:,:);
        load(j,:,:)=temp(:,:)+tempBC*A(j,l);
    end
end

