function [ A,B,C,ref ] = multiPF( X,varargin )
%multiPF Summary of this function goes here
%Step 1 of Parallel Factor Framework-Clustering Analysis (PFFCA):
%Data decompose
% ----------------------INPUT---------------------
%
% X            X is the input array, which should be three-dimensional data. 
%
% ----------------OPTIONAL INPUT---------------------
% 
% minFactor    minimum factor number allowed to be try. Default value is 1.
% 
% maxFactor    maximum factor number allowed to be try. Default value is 50.
% 
% initFactor   Initial factor number, if initFactor is less than minFactor
%              initFactor will be equal to minFactor. Default value is 1.
%
% fixFactor    Only the given factor number would be used to decompose data.
%              Default value is 0 (not fix). if fixFactor is used, the 
%              minFactor and maxFactor will be disabled.
% 
% threshold    The threshold is a number we set to indicate efficiently. 
%              If the RSS reduction rate (RSS0-RSS1)/RSS0 is beyond the threshold, 
%              the increase in factors can improve the model efficiently.
%              Default value is 0.05. 
% 
% Options      Optional parameters for parafac.
%
% ---------------------OUTPUT---------------------
%
% A, B, C      Estimate of loadings. 
%
% ref.log      Record the factor number, the sum of squares of errors and 
%              number of iterations used.
%
% ---------------------Example---------------------
%
%   [ A,B,C,ref ] = multiPF( X,'minFactor',3,'minFactor',30 );
%   decompose the data form 3 factor model, the output model's 
%   factor number less than 30.
%

% Chen Qian
% University of Science and Technology of China
% E-mail qianc@mail.ustc.edu.cn

%Default value of parameters
initFactor=1;
minFactor=1;
maxFactor=50;
fixFactor=0;
threshold=0.05;
iterateLimit=false;
Options(5)=100;
Options(6)=500;
Options(2)=0; % DTLD init
% Options(2)=1; % SVD init
% Options(2)=2; % random

while length(varargin)>=2,
    prop =varargin{1};
    val=varargin{2};
    varargin=varargin(3:end);
    eval([prop '=val;']);     
end

if fixFactor~=0
    initFactor=fixFactor;
    minFactor=fixFactor;
    maxFactor=fixFactor;
    iterateLimit=true;
    Options(6)=800;
end

if initFactor<minFactor
    initFactor=minFactor;
end

% init
factorNum=initFactor;
err0=inf;
A0=[];
ref.log=[];
secondTime=false;

% Decompose
while factorNum<=maxFactor
    [Factors,it,err]=parafac(X,factorNum,Options,[2 2 2]);%,1e-6
    ref.log=[ref.log;factorNum,err,it];
    [A,B,C]=fac2let(Factors);
    if err0-err<threshold*err0
        if secondTime==false
            secondTime=true;
            factorNum=factorNum+1;
            continue;
        else
            A=A0;B=B0;C=C0;err=err0;
            break;
        end
    elseif iterateLimit==true && it==Options(6)
        if factorNum==minFactor
            break;
        elseif isempty(A0) 
            factorNum=factorNum-1;
            maxFactor=factorNum;
            continue;
        else
            A=A0;B=B0;C=C0;err=err0;
            break;
        end
    else
        disp([num2str(factorNum),':',num2str(err)]);
        secondTime=false;
        A0=A;B0=B;C0=C;
        err0=err;
        factorNum=factorNum+1;
    end
end

% ref.err=err/size(A,2);
disp('factorNum calculation complete');
end

